// -*- C++ -*-
/*!
 * @file  Danger.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "Danger.h"
#include "type.h"

#include <iostream>
using namespace std;

// Module specification
// <rtc-template block="module_spec">
static const char* danger_spec[] =
  {
    "implementation_id", "Danger",
    "type_name",         "Danger",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "TUS",
    "category",          "Category",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.scan_range_distance", "750",
    "conf.default.scan_range_maxpoint", "720",
    "conf.default.scan_range_minpoint", "360",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Danger::Danger(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_UrgDataInIn("UrgDataIn", m_UrgDataIn),
    m_VelocityInIn("VelocityIn", m_VelocityIn),
    m_VelocityOutOut("VelocityOut", m_VelocityOut)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
Danger::~Danger()
{
}



RTC::ReturnCode_t Danger::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("UrgDataIn", m_UrgDataInIn);
  registerInPort("VelocityIn", m_VelocityInIn);
  
  // Set OutPort buffer
  registerOutPort("VelocityOut", m_VelocityOutOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("scan_range_distance", m_scan_range_distance, "750");
  bindParameter("scan_range_maxpoint", m_scan_range_maxpoint, "720");
  bindParameter("scan_range_minpoint", m_scan_range_minpoint, "360");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Danger::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Danger::onActivated(RTC::UniqueId ec_id)
{
	PortServiceList* portlist;
	portlist = this->get_ports();
	port_connect_check=new bool [portlist->length()];

	for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
	{
		PortService_ptr port;
		port = (*portlist)[i];
		cout <<"PortNum"<<i<<":"<<port->get_port_profile()->name<<"=";

		if(port->get_port_profile()->connector_profiles.length()!=0){
			port_connect_check[i]=true;
			cout <<"Connect"<<endl;
		}

		else if(port->get_port_profile()->connector_profiles.length()==0){
			port_connect_check[i]=false;
			cout <<"DisConnect"<<endl;
		}
	}

	cout <<"-----------------------"<<endl;


	lrf_scanpoint=NUMBER;
	lrf_resolusion=2*PI/CYCLE;
	lrf_offset=(-45)/180.0*PI;

	g_distance=new long int[lrf_scanpoint];
  return RTC::RTC_OK;
}


RTC::ReturnCode_t Danger::onDeactivated(RTC::UniqueId ec_id)
{
	delete[] g_distance;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t Danger::onExecute(RTC::UniqueId ec_id)
{
	static double speed,turn;
	int danger_flag=0;

	if(m_UrgDataInIn.isNew()){
		m_UrgDataInIn.read();
		if(m_UrgDataIn.data.length()==lrf_scanpoint){	// データ長チェック(符号なし)
			for(int i=0;i<lrf_scanpoint;i++){
				g_distance[i]=m_UrgDataIn.data[i];
			} 
		}


	}

	if (m_VelocityInIn.isNew()) {
		m_VelocityInIn.read();	// 速度情報読み込み
		speed=m_VelocityIn.v;
		turn=m_VelocityIn.w;
	}

	double m_number=m_scan_range_maxpoint - m_scan_range_minpoint;


	urg_danger(m_number,g_distance,m_scan_range_distance,m_scan_range_minpoint,m_scan_range_maxpoint,danger_flag);	// danger_chech.cpp参照

	if(danger_flag==1){	// 完全停止
		cout <<"Stop"<<endl;
		speed=0;
		turn=0;
	}

	else{	// 通常走行
		cout <<"Run"<<endl;	// danger_flag==0
	}

	m_VelocityOut.v=speed;
	m_VelocityOut.w=turn;
	m_VelocityOutOut.write();	// 速度情報上書き

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Danger::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Danger::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DangerInit(RTC::Manager* manager)
  {
    coil::Properties profile(danger_spec);
    manager->registerFactory(profile,
                             RTC::Create<Danger>,
                             RTC::Delete<Danger>);
  }
  
};


